@@ -3,11 +3,23 @@
 import click
 
 from .. import version
-from ..constants import (LOG_FILE_MAX_FILES, LOG_FILE_MAX_MB, LOG_FILE_NAME,
-                         LOG_FILE_PATH, LOG_LEVEL_API, LOG_LEVEL_AUTH,
-                         LOG_LEVEL_CONSOLE, LOG_LEVEL_FILE, LOG_LEVEL_HTTP,
-                         LOG_LEVEL_PACKAGE, LOG_LEVELS_STR, REQUEST_ATTR_MAP,
-                         RESPONSE_ATTR_MAP, TIMEOUT_CONNECT, TIMEOUT_RESPONSE)
+from ..constants import (
+    LOG_FILE_MAX_FILES,
+    LOG_FILE_MAX_MB,
+    LOG_FILE_NAME,
+    LOG_FILE_PATH,
+    LOG_LEVEL_API,
+    LOG_LEVEL_AUTH,
+    LOG_LEVEL_CONSOLE,
+    LOG_LEVEL_FILE,
+    LOG_LEVEL_HTTP,
+    LOG_LEVEL_PACKAGE,
+    LOG_LEVELS_STR,
+    REQUEST_ATTR_MAP,
+    RESPONSE_ATTR_MAP,
+    TIMEOUT_CONNECT,
+    TIMEOUT_RESPONSE,
+)
 from . import context, grp_adapters, grp_assets, grp_system, grp_tools
 
 
@@ -92,6 +104,7 @@ All of the options listed above must be supplied BEFORE any commands or groups.
     "-reqattr",
     "log_request_attrs",
     help="Log http client request attributes.",
+    default=["size", "url"],
     multiple=True,
     type=click.Choice(list(REQUEST_ATTR_MAP) + ["all"]),
     show_envvar=True,
@@ -100,6 +113,7 @@ All of the options listed above must be supplied BEFORE any commands or groups.
     "--log-response-attrs",
     "-respattr",
     "log_response_attrs",
+    default=["size", "url", "status", "elapsed"],
     help="Log http client response attributes.",
     multiple=True,
     type=click.Choice(list(RESPONSE_ATTR_MAP) + ["all"]),
@@ -124,8 +138,8 @@ All of the options listed above must be supplied BEFORE any commands or groups.
     show_envvar=True,
 )
 @click.option(
-    "--log-console",
-    "-c",
+    "--log-console/--no-log-console",
+    "-c/-nc",
     "log_console",
     default=False,
     help="Enable logging to STDERR.",
@@ -133,10 +147,10 @@ All of the options listed above must be supplied BEFORE any commands or groups.
     show_envvar=True,
 )
 @click.option(
-    "--log-file",
-    "-f",
+    "--log-file/--no-log-file",
+    "-f/-nf",
     "log_file",
-    default=False,
+    default=True,
     help="Enable logging to -fn/--log-file-name in -fp/--log-file-path.",
     is_flag=True,
     show_envvar=True,
@@ -157,7 +171,7 @@ All of the options listed above must be supplied BEFORE any commands or groups.
     "log_file_path",
     metavar="PATH",
     default=LOG_FILE_PATH,
-    help="Directory to use for -fn/--log-file-name (Defaults to CWD).",
+    help="Directory to use for -fn/--log-file-name (Defaults to current directory).",
     show_envvar=True,
 )
 @click.option(
